package gosimplenpm

import (
	"fmt"
	"gosimplenpm/internal/config"
	"gosimplenpm/internal/handler"
	"os"

	"github.com/sirupsen/logrus"
	"github.com/spf13/cobra"
)

var rootCmd = &cobra.Command{
	Use:   "gosimplenpm",
	Short: "Gosimplenpm is an implementation of npm registry server",
	Long: `Gosimplenpm is an implemenation of npm registry server.

It creates the repository to host all your private npm repos. It can
serve the npm packages offline. 

Documentation about the npm private registry: 
	https://docs.npmjs.com/packages-and-modules
`,
	Run: func(_ *cobra.Command, _ []string) {
		err := config.VerifyConfig()
		if err != nil {
			fmt.Printf("Error verifying config: %+v\n", err)
			os.Exit(1)
		}
		var cfg config.Config
		err = config.LoadOrCreateConfig(&cfg)
		if err != nil {
			fmt.Printf("Error loading config: %+v\n", err)
			os.Exit(1)
		}
		lvl, err := logrus.ParseLevel(cfg.LogLevel)
		if err != nil {
			fmt.Printf("%+v", err)
			os.Exit(1)
		}
		log := &logrus.Logger{
			Out:   os.Stdout,
			Level: lvl,
			Formatter: &logrus.TextFormatter{
				FullTimestamp:   true,
				TimestampFormat: "2009-01-02 15:15:15",
			},
		}
		app := &handler.Application{
			Conf:   cfg,
			Logger: log,
		}

		fmt.Println("\n Server is starting....")
		err = app.Start()
		fmt.Println("Why!")
		if err != nil {
			fmt.Printf("Server start up error: %+v\n", err)
			os.Exit(1)
		}
	},
}

func Execute() {
	err := rootCmd.Execute()
	if err != nil {
		os.Exit(1)
	}
}

func init() {
	rootCmd.CompletionOptions.HiddenDefaultCmd = true
	// hide help command
	rootCmd.SetHelpCommand(&cobra.Command{Hidden: true})
	// hide help flag
	rootCmd.PersistentFlags().BoolP("help", "h", false, "This help")
	rootCmd.PersistentFlags().Lookup("help").Hidden = true
	// TODO: get some variables from flags
	rootCmd.Flags().StringVarP(&config.HTTPListen, "listen", "l", config.HTTPListen, "HTTP bind interface and port for server")
	_ = rootCmd.MarkFlagRequired("listen")
	rootCmd.Flags().BoolVarP(&config.CanLog, "verbose", "v", config.CanLog, "Logging level for server")
	rootCmd.Flags().StringVar(&config.NpmRepoDir, "repodir", config.NpmRepoDir, "Repo dir to house published packages")
}
