package handler

import (
	"encoding/json"
	"fmt"
	"gosimplenpm/internal/config"
	"gosimplenpm/internal/serviceidos"
	"gosimplenpm/internal/storage"
	"net/http"
	"net/url"
	"strconv"

	"github.com/gorilla/mux"
	"github.com/sirupsen/logrus"
)

func DistTagGet(lg *logrus.Logger, cfg config.Config) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		escapedName := mux.Vars(r)["name"]
		packageName, _ := url.PathUnescape(escapedName)
		lg.WithFields(logrus.Fields{
			"function": "dist-tags-get",
		}).Debugf("Package name => %s\n", packageName)

		fileToServe, found, err := storage.GetIndexJsonFromStore(packageName, cfg.RepoDir, lg)
		if err != nil {
			http.Error(w, err.Error(), http.StatusInternalServerError)
			return
		}

		if !found {
			ret := fmt.Sprintf("Package not found: %s", packageName)
			http.Error(w, ret, http.StatusNotFound)
			return
		}

		var jsonFile serviceidos.IndexJson
		err = storage.ReadIndexJson(fileToServe, &jsonFile, lg)
		if err != nil {
			http.Error(w, err.Error(), http.StatusInternalServerError)
			return
		}

		jsonString, _ := json.Marshal(jsonFile.DistTags)

		w.Header().Set("Content-Type", "application/json")
		w.Header().Set("Content-Length", strconv.Itoa(len(jsonString)))
		w.Write(jsonString)
	}
}
